// HSS.IO.MimeMapping.cs
// ----------------------------------------------------------------------------
// Licensed under the MIT license
// http://www.opensource.org/licenses/mit-license.html
// ----------------------------------------------------------------------------
// HighSpeed-Solutions, LLC
// Copyright (c) 2001-2010
// ----------------------------------------------------------------------------
// File:       MimeMapping.cs
// Author:     HSS\gbanta
// Created:    08/12/2010
// Modified:   12/04/2010
// ----------------------------------------------------------------------------
namespace HSS.IO
{
	#region Using Directives
	using System;
	using System.Collections.Generic;
	using System.Security.Permissions;
	using Microsoft.Win32;
	#endregion

	#region MimeMapping
	/// <summary>
	/// Supports retrieving, and registering in memory custom MIME types, for a given file type.
	/// </summary>
	public static class MimeMapping
	{
		#region Constants
		private const string defaultExt = ".*";
		private const string defaultMIMEType = "application/octet-stream";
		#endregion

		#region Fields
		private static Dictionary<string, string> maps;
		#endregion

		#region Constructors
		static MimeMapping()
		{
			#region Default Mappings
			MimeMapping.maps = new Dictionary<string, string>(190, StringComparer.CurrentCultureIgnoreCase);
			MimeMapping.AddMimeMapping(".323", "text/h323");
			MimeMapping.AddMimeMapping(".asx", "video/x-ms-asf");
			MimeMapping.AddMimeMapping(".acx", "application/internet-property-stream");
			MimeMapping.AddMimeMapping(".ai", "application/postscript");
			MimeMapping.AddMimeMapping(".aif", "audio/x-aiff");
			MimeMapping.AddMimeMapping(".aiff", "audio/aiff");
			MimeMapping.AddMimeMapping(".axs", "application/olescript");
			MimeMapping.AddMimeMapping(".aifc", "audio/aiff");
			MimeMapping.AddMimeMapping(".asr", "video/x-ms-asf");
			MimeMapping.AddMimeMapping(".avi", "video/x-msvideo");
			MimeMapping.AddMimeMapping(".asf", "video/x-ms-asf");
			MimeMapping.AddMimeMapping(".au", "audio/basic");
			MimeMapping.AddMimeMapping(".application", "application/x-ms-application");
			MimeMapping.AddMimeMapping(".bin", "application/octet-stream");
			MimeMapping.AddMimeMapping(".bas", "text/plain");
			MimeMapping.AddMimeMapping(".bcpio", "application/x-bcpio");
			MimeMapping.AddMimeMapping(".bmp", "image/bmp");
			MimeMapping.AddMimeMapping(".cdf", "application/x-cdf");
			MimeMapping.AddMimeMapping(".cat", "application/vndms-pkiseccat");
			MimeMapping.AddMimeMapping(".crt", "application/x-x509-ca-cert");
			MimeMapping.AddMimeMapping(".c", "text/plain");
			MimeMapping.AddMimeMapping(".css", "text/css");
			MimeMapping.AddMimeMapping(".cer", "application/x-x509-ca-cert");
			MimeMapping.AddMimeMapping(".crl", "application/pkix-crl");
			MimeMapping.AddMimeMapping(".cmx", "image/x-cmx");
			MimeMapping.AddMimeMapping(".csh", "application/x-csh");
			MimeMapping.AddMimeMapping(".cod", "image/cis-cod");
			MimeMapping.AddMimeMapping(".cpio", "application/x-cpio");
			MimeMapping.AddMimeMapping(".clp", "application/x-msclip");
			MimeMapping.AddMimeMapping(".crd", "application/x-mscardfile");
			MimeMapping.AddMimeMapping(".deploy", "application/octet-stream");
			MimeMapping.AddMimeMapping(".dll", "application/x-msdownload");
			MimeMapping.AddMimeMapping(".dot", "application/msword");
			MimeMapping.AddMimeMapping(".doc", "application/msword");
			MimeMapping.AddMimeMapping(".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document");
			MimeMapping.AddMimeMapping(".dotx", "application/vnd.openxmlformats-officedocument.wordprocessingml.template");
			MimeMapping.AddMimeMapping(".docm", "application/vnd.ms-word.document.macroEnabled.12");
			MimeMapping.AddMimeMapping(".dotm", "application/vnd.ms-word.template.macroEnabled.12");
			MimeMapping.AddMimeMapping(".dvi", "application/x-dvi");
			MimeMapping.AddMimeMapping(".dir", "application/x-director");
			MimeMapping.AddMimeMapping(".dxr", "application/x-director");
			MimeMapping.AddMimeMapping(".der", "application/x-x509-ca-cert");
			MimeMapping.AddMimeMapping(".dib", "image/bmp");
			MimeMapping.AddMimeMapping(".dcr", "application/x-director");
			MimeMapping.AddMimeMapping(".disco", "text/xml");
			MimeMapping.AddMimeMapping(".exe", "application/octet-stream");
			MimeMapping.AddMimeMapping(".etx", "text/x-setext");
			MimeMapping.AddMimeMapping(".evy", "application/envoy");
			MimeMapping.AddMimeMapping(".eml", "message/rfc822");
			MimeMapping.AddMimeMapping(".eps", "application/postscript");
			MimeMapping.AddMimeMapping(".flr", "x-world/x-vrml");
			MimeMapping.AddMimeMapping(".fif", "application/fractals");
			MimeMapping.AddMimeMapping(".gtar", "application/x-gtar");
			MimeMapping.AddMimeMapping(".gif", "image/gif");
			MimeMapping.AddMimeMapping(".gz", "application/x-gzip");
			MimeMapping.AddMimeMapping(".hta", "application/hta");
			MimeMapping.AddMimeMapping(".htc", "text/x-component");
			MimeMapping.AddMimeMapping(".htt", "text/webviewhtml");
			MimeMapping.AddMimeMapping(".h", "text/plain");
			MimeMapping.AddMimeMapping(".hdf", "application/x-hdf");
			MimeMapping.AddMimeMapping(".hlp", "application/winhlp");
			MimeMapping.AddMimeMapping(".html", "text/html");
			MimeMapping.AddMimeMapping(".htm", "text/html");
			MimeMapping.AddMimeMapping(".hqx", "application/mac-binhex40");
			MimeMapping.AddMimeMapping(".isp", "application/x-internet-signup");
			MimeMapping.AddMimeMapping(".iii", "application/x-iphone");
			MimeMapping.AddMimeMapping(".ief", "image/ief");
			MimeMapping.AddMimeMapping(".ivf", "video/x-ivf");
			MimeMapping.AddMimeMapping(".ins", "application/x-internet-signup");
			MimeMapping.AddMimeMapping(".ico", "image/x-icon");
			MimeMapping.AddMimeMapping(".jpg", "image/jpeg");
			MimeMapping.AddMimeMapping(".jfif", "image/pjpeg");
			MimeMapping.AddMimeMapping(".jpe", "image/jpeg");
			MimeMapping.AddMimeMapping(".jpeg", "image/jpeg");
			MimeMapping.AddMimeMapping(".js", "application/x-javascript");
			MimeMapping.AddMimeMapping(".lsx", "video/x-la-asf");
			MimeMapping.AddMimeMapping(".latex", "application/x-latex");
			MimeMapping.AddMimeMapping(".lsf", "video/x-la-asf");
			MimeMapping.AddMimeMapping(".manifest", "application/x-ms-manifest");
			MimeMapping.AddMimeMapping(".mhtml", "message/rfc822");
			MimeMapping.AddMimeMapping(".mny", "application/x-msmoney");
			MimeMapping.AddMimeMapping(".mht", "message/rfc822");
			MimeMapping.AddMimeMapping(".mid", "audio/mid");
			MimeMapping.AddMimeMapping(".mpv2", "video/mpeg");
			MimeMapping.AddMimeMapping(".man", "application/x-troff-man");
			MimeMapping.AddMimeMapping(".mvb", "application/x-msmediaview");
			MimeMapping.AddMimeMapping(".mpeg", "video/mpeg");
			MimeMapping.AddMimeMapping(".m3u", "audio/x-mpegurl");
			MimeMapping.AddMimeMapping(".mdb", "application/x-msaccess");
			MimeMapping.AddMimeMapping(".mpp", "application/vnd.ms-project");
			MimeMapping.AddMimeMapping(".m1v", "video/mpeg");
			MimeMapping.AddMimeMapping(".mpa", "video/mpeg");
			MimeMapping.AddMimeMapping(".me", "application/x-troff-me");
			MimeMapping.AddMimeMapping(".m13", "application/x-msmediaview");
			MimeMapping.AddMimeMapping(".movie", "video/x-sgi-movie");
			MimeMapping.AddMimeMapping(".m14", "application/x-msmediaview");
			MimeMapping.AddMimeMapping(".mpe", "video/mpeg");
			MimeMapping.AddMimeMapping(".mp2", "video/mpeg");
			MimeMapping.AddMimeMapping(".mov", "video/quicktime");
			MimeMapping.AddMimeMapping(".mp3", "audio/mpeg");
			MimeMapping.AddMimeMapping(".mpg", "video/mpeg");
			MimeMapping.AddMimeMapping(".ms", "application/x-troff-ms");
			MimeMapping.AddMimeMapping(".nc", "application/x-netcdf");
			MimeMapping.AddMimeMapping(".nws", "message/rfc822");
			MimeMapping.AddMimeMapping(".oda", "application/oda");
			MimeMapping.AddMimeMapping(".ods", "application/oleobject");
			MimeMapping.AddMimeMapping(".pmc", "application/x-perfmon");
			MimeMapping.AddMimeMapping(".p7r", "application/x-pkcs7-certreqresp");
			MimeMapping.AddMimeMapping(".p7b", "application/x-pkcs7-certificates");
			MimeMapping.AddMimeMapping(".p7s", "application/pkcs7-signature");
			MimeMapping.AddMimeMapping(".pmw", "application/x-perfmon");
			MimeMapping.AddMimeMapping(".ps", "application/postscript");
			MimeMapping.AddMimeMapping(".p7c", "application/pkcs7-mime");
			MimeMapping.AddMimeMapping(".pbm", "image/x-portable-bitmap");
			MimeMapping.AddMimeMapping(".png", "image/png");
			MimeMapping.AddMimeMapping(".ppm", "image/x-portable-pixmap");
			MimeMapping.AddMimeMapping(".pub", "application/x-mspublisher");
			MimeMapping.AddMimeMapping(".pnm", "image/x-portable-anymap");
			MimeMapping.AddMimeMapping(".pml", "application/x-perfmon");
			MimeMapping.AddMimeMapping(".p10", "application/pkcs10");
			MimeMapping.AddMimeMapping(".pfx", "application/x-pkcs12");
			MimeMapping.AddMimeMapping(".p12", "application/x-pkcs12");
			MimeMapping.AddMimeMapping(".pdf", "application/pdf");
			MimeMapping.AddMimeMapping(".pps", "application/vnd.ms-powerpoint");
			MimeMapping.AddMimeMapping(".p7m", "application/pkcs7-mime");
			MimeMapping.AddMimeMapping(".pko", "application/vndms-pkipko");
			MimeMapping.AddMimeMapping(".ppt", "application/vnd.ms-powerpoint");
			MimeMapping.AddMimeMapping(".pptx", "application/vnd.openxmlformats-officedocument.presentationml.presentation");
			MimeMapping.AddMimeMapping(".potx", "application/vnd.openxmlformats-officedocument.presentationml.template");
			MimeMapping.AddMimeMapping(".ppsx", "application/vnd.openxmlformats-officedocument.presentationml.slideshow");
			MimeMapping.AddMimeMapping(".ppam", "application/vnd.ms-powerpoint.addin.macroEnabled.12");
			MimeMapping.AddMimeMapping(".pptm", "application/vnd.ms-powerpoint.presentation.macroEnabled.12");
			MimeMapping.AddMimeMapping(".potm", "application/vnd.ms-powerpoint.presentation.macroEnabled.12");
			MimeMapping.AddMimeMapping(".ppsm", "application/vnd.ms-powerpoint.slideshow.macroEnabled.12");
			MimeMapping.AddMimeMapping(".pmr", "application/x-perfmon");
			MimeMapping.AddMimeMapping(".pma", "application/x-perfmon");
			MimeMapping.AddMimeMapping(".pot", "application/vnd.ms-powerpoint");
			MimeMapping.AddMimeMapping(".prf", "application/pics-rules");
			MimeMapping.AddMimeMapping(".pgm", "image/x-portable-graymap");
			MimeMapping.AddMimeMapping(".qt", "video/quicktime");
			MimeMapping.AddMimeMapping(".ra", "audio/x-pn-realaudio");
			MimeMapping.AddMimeMapping(".rgb", "image/x-rgb");
			MimeMapping.AddMimeMapping(".ram", "audio/x-pn-realaudio");
			MimeMapping.AddMimeMapping(".rmi", "audio/mid");
			MimeMapping.AddMimeMapping(".ras", "image/x-cmu-raster");
			MimeMapping.AddMimeMapping(".roff", "application/x-troff");
			MimeMapping.AddMimeMapping(".rtf", "application/rtf");
			MimeMapping.AddMimeMapping(".rtx", "text/richtext");
			MimeMapping.AddMimeMapping(".sv4crc", "application/x-sv4crc");
			MimeMapping.AddMimeMapping(".spc", "application/x-pkcs7-certificates");
			MimeMapping.AddMimeMapping(".setreg", "application/set-registration-initiation");
			MimeMapping.AddMimeMapping(".snd", "audio/basic");
			MimeMapping.AddMimeMapping(".stl", "application/vndms-pkistl");
			MimeMapping.AddMimeMapping(".setpay", "application/set-payment-initiation");
			MimeMapping.AddMimeMapping(".stm", "text/html");
			MimeMapping.AddMimeMapping(".shar", "application/x-shar");
			MimeMapping.AddMimeMapping(".sh", "application/x-sh");
			MimeMapping.AddMimeMapping(".sit", "application/x-stuffit");
			MimeMapping.AddMimeMapping(".spl", "application/futuresplash");
			MimeMapping.AddMimeMapping(".sct", "text/scriptlet");
			MimeMapping.AddMimeMapping(".scd", "application/x-msschedule");
			MimeMapping.AddMimeMapping(".sst", "application/vndms-pkicertstore");
			MimeMapping.AddMimeMapping(".src", "application/x-wais-source");
			MimeMapping.AddMimeMapping(".sv4cpio", "application/x-sv4cpio");
			MimeMapping.AddMimeMapping(".tex", "application/x-tex");
			MimeMapping.AddMimeMapping(".tgz", "application/x-compressed");
			MimeMapping.AddMimeMapping(".t", "application/x-troff");
			MimeMapping.AddMimeMapping(".tar", "application/x-tar");
			MimeMapping.AddMimeMapping(".tr", "application/x-troff");
			MimeMapping.AddMimeMapping(".tif", "image/tiff");
			MimeMapping.AddMimeMapping(".txt", "text/plain");
			MimeMapping.AddMimeMapping(".texinfo", "application/x-texinfo");
			MimeMapping.AddMimeMapping(".trm", "application/x-msterminal");
			MimeMapping.AddMimeMapping(".tiff", "image/tiff");
			MimeMapping.AddMimeMapping(".tcl", "application/x-tcl");
			MimeMapping.AddMimeMapping(".texi", "application/x-texinfo");
			MimeMapping.AddMimeMapping(".tsv", "text/tab-separated-values");
			MimeMapping.AddMimeMapping(".ustar", "application/x-ustar");
			MimeMapping.AddMimeMapping(".uls", "text/iuls");
			MimeMapping.AddMimeMapping(".vcf", "text/x-vcard");
			MimeMapping.AddMimeMapping(".wps", "application/vnd.ms-works");
			MimeMapping.AddMimeMapping(".wav", "audio/wav");
			MimeMapping.AddMimeMapping(".wrz", "x-world/x-vrml");
			MimeMapping.AddMimeMapping(".wri", "application/x-mswrite");
			MimeMapping.AddMimeMapping(".wks", "application/vnd.ms-works");
			MimeMapping.AddMimeMapping(".wmf", "application/x-msmetafile");
			MimeMapping.AddMimeMapping(".wcm", "application/vnd.ms-works");
			MimeMapping.AddMimeMapping(".wrl", "x-world/x-vrml");
			MimeMapping.AddMimeMapping(".wdb", "application/vnd.ms-works");
			MimeMapping.AddMimeMapping(".wsdl", "text/xml");
			MimeMapping.AddMimeMapping(".xaml", "application/xaml+xml");
			MimeMapping.AddMimeMapping(".xml", "text/xml");
			MimeMapping.AddMimeMapping(".xlm", "application/vnd.ms-excel");
			MimeMapping.AddMimeMapping(".xaf", "x-world/x-vrml");
			MimeMapping.AddMimeMapping(".xla", "application/vnd.ms-excel");
			MimeMapping.AddMimeMapping(".xls", "application/vnd.ms-excel");
			MimeMapping.AddMimeMapping(".xof", "x-world/x-vrml");
			MimeMapping.AddMimeMapping(".xlt", "application/vnd.ms-excel");
			MimeMapping.AddMimeMapping(".xlc", "application/vnd.ms-excel");
			MimeMapping.AddMimeMapping(".xlsx", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
			MimeMapping.AddMimeMapping(".xltx", "application/vnd.openxmlformats-officedocument.spreadsheetml.template");
			MimeMapping.AddMimeMapping(".xlsm", "application/vnd.ms-excel.sheet.macroEnabled.12");
			MimeMapping.AddMimeMapping(".xltm", "application/vnd.ms-excel.template.macroEnabled.12");
			MimeMapping.AddMimeMapping(".xlam", "application/vnd.ms-excel.addin.macroEnabled.12");
			MimeMapping.AddMimeMapping(".xlsb", "application/vnd.ms-excel.sheet.binary.macroEnabled.12");
			MimeMapping.AddMimeMapping(".xsl", "text/xml");
			MimeMapping.AddMimeMapping(".xbm", "image/x-xbitmap");
			MimeMapping.AddMimeMapping(".xlw", "application/vnd.ms-excel");
			MimeMapping.AddMimeMapping(".xpm", "image/x-xpixmap");
			MimeMapping.AddMimeMapping(".xwd", "image/x-xwindowdump");
			MimeMapping.AddMimeMapping(".xsd", "text/xml");
			MimeMapping.AddMimeMapping(".z", "application/x-compress");
			MimeMapping.AddMimeMapping(".zip", "application/x-zip-compressed");
			MimeMapping.AddMimeMapping(defaultExt, "application/octet-stream");
			#endregion

			#region Update/override Mappings from Registry.
			try
			{
				RegistryPermission regPerm = new RegistryPermission(RegistryPermissionAccess.Read, "HKEY_CLASSES_ROOT");
				regPerm.Assert();
				try
				{
					foreach (string extensionKey in Microsoft.Win32.Registry.ClassesRoot.GetSubKeyNames())
					{
						if (extensionKey.StartsWith("."))
						{
							RegistryKey regKey = Registry.ClassesRoot.OpenSubKey(extensionKey);
							if (regKey != null)
							{
								object contentType = regKey.GetValue("Content Type");
								if (contentType != null)
								{
									string extension = extensionKey.ToLowerInvariant();
									string mimeType = contentType.ToString();
									if (!MimeMapping.maps.ContainsKey(extension))
										MimeMapping.maps.Add(extension, mimeType);
									else
										MimeMapping.maps[extension] = mimeType;
								}
							}
						}
					}
				}
				catch (Exception ex)
				{
					HSS.Logging.Log.WriteException(ex);
				}
				finally
				{
					RegistryPermission.RevertAssert();
				}
			}
			catch (Exception ex)
			{
				HSS.Logging.Log.WriteException(ex);
			}
			#endregion
		}
		#endregion

		#region Methods
		/// <summary>
		/// Add a File Extension and MIME Type to the MIME Mapping. Note, the Map is already pre-filled with all the standard MIME Types and File Extensions
		/// </summary>
		/// <param name="extension">The extension of the file for the given MIME Type, including the dot (example: .ext).</param>
		/// <param name="mimeType">The MIME Type that maps to the provided file extension</param>
		/// <returns>true if the extension was added; otherwise false.</returns>
		public static bool AddMimeMapping(string extension, string mimeType)
		{
			if (!MimeMapping.maps.ContainsKey(extension))
			{
				MimeMapping.maps.Add(extension, mimeType);
				return true;
			}
			return false;
		}
		/// <summary>
		/// Retrieve the MIME type for a given file name
		/// </summary>
		/// <param name="fileName">The name of the file including extension (example: file.ext)</param>
		/// <returns>The MIME Type found; otherwise 'application/octet-stream'.</returns>
		public static string FromFileName(string fileName)
		{
			string fileExtension = System.IO.Path.GetExtension(fileName);
			return FromFileExtension(fileExtension);
		}
		/// <summary>
		/// Retrieve the MIME type for a given file extension
		/// </summary>
		/// <param name="fileExtension">The extension of file (example: .ext)</param>
		/// <returns>The MIME Type found; otherwise 'application/octet-stream'.</returns>
		public static string FromFileExtension(string fileExtension)
		{
			string mimeType = null;
			if (maps.TryGetValue(fileExtension, out mimeType))
				return mimeType;
			return defaultMIMEType;
		}
		#endregion
	}
	#endregion
}