// HSS.Forms.BrowseModuleContainer.cs
// ----------------------------------------------------------------------------
// Licensed under the MIT license
// http://www.opensource.org/licenses/mit-license.html
// ----------------------------------------------------------------------------
// HighSpeed-Solutions, LLC
// Copyright (c) 2001-2010
// ----------------------------------------------------------------------------
// File:       BrowseModuleContainer.cs
// Author:     HSS\gbanta
// Created:    08/12/2010
// Modified:   12/23/2010
// ----------------------------------------------------------------------------
namespace HSS.Forms
{
	#region Using Directives
	using System;
	using System.Collections.Generic;
	using System.ComponentModel;
	using System.Drawing;
	using System.Windows.Forms;
	using HSS.Data;

	#endregion

	#region BrowseModuleContainer
	/// <summary>
	/// Summary description for BrowseModuleContainer.
	/// </summary>
	public partial class BrowseModuleContainer : System.Windows.Forms.Form
	{
		static object lockObject = new object();

		/// <summary>
		/// Default Constructor
		/// </summary>
		public BrowseModuleContainer()
		{
			InitializeComponent();
		}

		#region Child Module Handlers

		/// <summary>
		/// The dynamically added Module
		/// </summary>
		public BrowseModuleBase ChildModule
		{
			get { return _childModule; }
			set { _childModule = value; }
		} BrowseModuleBase _childModule;

		/// <summary>
		/// Initializes the dynamically added Module
		/// </summary>
		/// <param name="objModule">The Module to add</param>
		/// <param name="KeyName">The name to associate with the Module</param>
		public void InitializeModuleContainer(BrowseModuleBase objModule, string KeyName)
		{
			if (null != objModule)
			{
				_childModule = objModule;
				_childModule.Name = KeyName;
				_childModule.Visible = true;
				_childModule.Dock = DockStyle.Fill;

				// Child Module's Parent Form Icon
				if (null != _childModule.ParentIcon)
					this.Icon = _childModule.ParentIcon;

				// Child Module - Image
				if (null != _childModule.Image)
					this.pbxModuleIcon.Image = _childModule.Image;

				this.cmdSelect.Enabled = false;

				// ShowInTaskbar
				this.ShowInTaskbar = _childModule.ShowInTaskbar;

				// Maximize
				this.MaximizeBox = _childModule.MaximizeBox;

				// Minimize
				this.MinimizeBox = _childModule.MinimizeBox;

				// FormBorderStyle
				this.FormBorderStyle = _childModule.ParentBorderStyle;

				// StartPosition
				this.StartPosition = _childModule.ParentStartPosition;

				// Size parent
				this.ResizeParent();

				this.pnlDetail.Controls.Add(_childModule);

			}
			else
				throw new ArgumentNullException("objModule", "The Find Module object [" + KeyName + "] is not instantiated or is null");
		}

		/// <summary>
		/// InitializeModule
		/// </summary>
		/// <param name="Data">An array of IDataContainer Objects to list</param>
		/// <param name="HeaderText">The text to display in the header</param>
		public bool InitializeModule(List<IDataContainer> Data, string HeaderText)
		{
			this.Text = HeaderText;

			// Header Visible
			if (_childModule.HeaderVisible)
			{
				this.pnlHeader.Height = _childModule.HeaderHeight;

				// Module Title
				if (!string.IsNullOrEmpty(_childModule.ModuleTitle))
					this.lblModuleTitle.Text = _childModule.ModuleTitle;
				else
					this.lblModuleTitle.Text = HeaderText;

				// Action Description
				if (!string.IsNullOrEmpty(_childModule.ActionDescription))
					this.lblActionDescription.Text = _childModule.ActionDescription;
				else
					this.lblActionDescription.Text = "Browse list and select the desired item.";

			}
			else
			{
				this.pnlHeader.Height = 0;
			}

			return _childModule.LoadData(Data);
		}


		/// <summary>
		/// Force a resize of the Parent Form (this)
		/// </summary>
		public void ResizeParent()
		{
			ResizeParent(_childModule.Size);
		}


		/// <summary>
		/// Force a resize of the Parent Form (this)
		/// </summary>
		/// <param name="NewSize">The minimum size for the Child Module</param>
		public void ResizeParent(Size NewSize)
		{
			if (this.WindowState == FormWindowState.Normal)
			{
				int newHeight =
					(this._childModule.HeaderVisible ? this._childModule.HeaderHeight : 0) +
					NewSize.Height +
					this.pnlFooter.Height;

				int newWidth = NewSize.Width;

				this.ClientSize = new Size(newWidth, newHeight);
			}
		}


		/// <summary>
		/// Indicates if this instance is the Active Instance
		/// </summary>
		public bool IsActive
		{
			get { return _isActive; }
			set { _isActive = value; }
		} bool _isActive = false;

		#endregion

		#region Properties
		/// <summary>
		/// The Title or Overview for the child Module
		/// </summary>
		[Description("The Title or Overview for the child Module")]
		[Category("Display")]
		public string ModuleTitle
		{
			get { return this.lblModuleTitle.Text; }
			set { this.lblModuleTitle.Text = value; }
		}
		/// <summary>
		/// The description or action for this child Module
		/// </summary>
		[Description("The description or action for this child Module")]
		[Category("Display")]
		public string ActionDescription
		{
			get { return this.lblActionDescription.Text; }
			set { this.lblActionDescription.Text = value; }
		}
		/// <summary>
		/// Is the Select Button enabled
		/// </summary>
		[Browsable(false)]
		public bool SelectButtonEnabled
		{
			get { return this.cmdSelect.Enabled; }
			set { this.cmdSelect.Enabled = value; }
		}
		#endregion

		#region Methods
		/// <summary>
		/// Executes the Cancel Action
		/// </summary>
		public virtual void PerformCancelAction()
		{
			this.DialogResult = DialogResult.Cancel;
			this.Close();
		}
		/// <summary>
		/// Executes the Selection Action
		/// </summary>
		public virtual void PerformSelectAction()
		{
			try
			{
				if (null != BrowseModuleContainer.Selected && this._childModule.RaiseSelectedEvent)
				{
					BrowseModuleContainer.Selected(this._childModule, EventArgs.Empty);
				}
				if (this._childModule.CloseOnSelected)
				{
					this.DialogResult = DialogResult.OK;
					this.Close();
				}
			}
			catch (Exception ex)
			{
				_childModule.MsgBox(ex.ToString());
			}
		}
		#endregion

		#region Events and Handlers
		private void cmdCancel_Click(object sender, System.EventArgs e)
		{
			PerformCancelAction();
		}
		private void cmdSelect_Click(object sender, System.EventArgs e)
		{
			PerformSelectAction();
		}

		private void BrowseModuleContainer_Activated(object sender, EventArgs e)
		{
			this.pnlFooter.Active = true;
			this._isActive = true;
		}
		private void BrowseModuleContainer_Deactivate(object sender, EventArgs e)
		{
			this._isActive = false;
			this.pnlFooter.Active = false;
		}
		private void BrowseModuleContainer_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (!e.Cancel)
				CloseInstance(this);
		}
		#endregion

		#region Static Helpers
		/// <summary>
		/// The number of open Configuration Editor forms
		/// </summary>
		public static int InstanceCount
		{
			get { return _instanceCount; }
			private set { _instanceCount = value; }
		} static int _instanceCount;
		/// <summary>
		/// Array of Configuration Editor Form Instances
		/// </summary>
		public static BrowseModuleContainer[] Instances()
		{
			BrowseModuleContainer[] local = new BrowseModuleContainer[_instances.Count];
			_instances.Values.CopyTo(local, 0);
			return local;
		} static Dictionary<string, BrowseModuleContainer> _instances = new Dictionary<string, BrowseModuleContainer>();
		/// <summary>
		/// Returns the active instance
		/// </summary>
		public static BrowseModuleContainer ActiveInstance
		{
			get
			{
				BrowseModuleContainer[] local = new BrowseModuleContainer[_instances.Count];
				_instances.Values.CopyTo(local, 0);

				foreach (BrowseModuleContainer instance in local)
				{
					if (instance._isActive)
					{
						return instance;
					}
				}
				return null;
			}
		}
		/// <summary>
		/// Returns the active instance specified by the name
		/// </summary>
		/// <param name="Name">The name of the Child Module</param>
		public static BrowseModuleContainer GetInstance(string Name)
		{
			BrowseModuleContainer[] local = new BrowseModuleContainer[_instances.Count];
			_instances.Values.CopyTo(local, 0);
			foreach (BrowseModuleContainer instance in local)
			{
				if (instance._childModule.Name == Name)
				{
					return instance;
				}
			}
			return null;
		}
		/// <summary>
		/// Load the Config Form
		/// </summary>
		/// <param name="typeName">The type within the assembly</param>
		/// <param name="FormTitle">The Title to display</param>
		/// <param name="Data">An array of IDataContainer Object to list</param>
		/// <param name="ShowAsDialog">Indicates if the instance should be shown as a Modal Dialog</param>
		/// <param name="owner">The Owning Parent Form</param>
		public static DialogResult LoadInstance(string typeName, string FormTitle, List<IDataContainer> Data, bool ShowAsDialog, IWin32Window owner)
		{
			BrowseModuleBase _childModule = null;
			DialogResult result = DialogResult.None;
			BrowseModuleContainer _detailForm = null;

			try
			{
				_detailForm = new BrowseModuleContainer();
				_detailForm.Name = "BrowseModuleContainer" + _detailForm.GetHashCode().ToString("x");
				_detailForm.IsActive = true;
				_instances.Add(_detailForm.Name, _detailForm);
				System.Threading.Interlocked.Increment(ref _instanceCount);

				// The Type to create
				Type t = Type.GetType(typeName, false);

				// Must have a valid Type to continue
				if (null == t)
					throw new InvalidCastException("Request Module Type [ " + typeName + " ] does not exist or is not available.");

				// Cast from BrowseModuleBase to the Instance Type Requested.
				_childModule = t.Assembly.CreateInstance(typeName) as BrowseModuleBase;

				// Initialize the Container
				_detailForm.InitializeModuleContainer(_childModule, typeName + _instanceCount.ToString());

				// Initialize the Module
				if (_detailForm.InitializeModule(Data, FormTitle))
				{
					if (ShowAsDialog)
					{
						result = _detailForm.ShowDialog(owner);
						if (null != _detailForm)
						{
							_detailForm.Dispose(true);
						}
					}
					else
					{
						_detailForm.Show(owner);
						_detailForm.BringToFront();
					}
				}
				else
				{
					CloseInstance(_detailForm);
					if (null != _detailForm)
					{
						_detailForm.Dispose(true);
					}
				}
			}
			catch (Exception ex)
			{
				CloseInstance(_detailForm);
				if (null != _detailForm)
				{
					_detailForm.Dispose(true);
				}
				MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			return result;
		}
		/// <summary>
		/// Closes a BrowseModuleContainer instance
		/// </summary>
		/// <param name="instance">A BrowseModuleContainer instance</param>
		private static void CloseInstance(BrowseModuleContainer instance)
		{
			if (null != instance)
			{
				try
				{
				}
				finally
				{
					try
					{
						string name = instance.Name;
						if (BrowseModuleContainer._instances.ContainsKey(name))
						{
							BrowseModuleContainer._instances.Remove(name);
							System.Threading.Interlocked.Decrement(ref _instanceCount);
						}
					}
					catch { }
				}
			}
		}
		/// <summary>
		/// Event to trigger when use has selected an item from the Child Control
		/// </summary>
		public static event EventHandler Selected;
		#endregion

	}
	#endregion
}