// HSS.Forms.Docking.AutoHideStripBase.cs
// ----------------------------------------------------------------------------
// Licensed under the MIT license
// http://www.opensource.org/licenses/mit-license.html
// ----------------------------------------------------------------------------
// HighSpeed-Solutions, LLC
// Copyright (c) 2001-2010
// ----------------------------------------------------------------------------
// File:       AutoHideStripBase.cs
// Author:     HSS\gbanta
// Created:    08/12/2010
// Modified:   12/04/2010
// ----------------------------------------------------------------------------
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace HSS.Forms.Docking
{
	/// <summary>
	/// AutoHideStripBase
	/// </summary>
	public abstract partial class AutoHideStripBase : Control
	{
		#region Tab
		/// <summary>
		/// Tab
		/// </summary>
		[SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible")]
        protected class Tab : IDisposable
        {
            private IDockContent _content;

			/// <summary>
			/// Tab from Content
			/// </summary>
			/// <param name="content">IDockContent</param>
            protected internal Tab(IDockContent content)
            {
                _content = content;
            }

			/// <summary>
			/// Finalizer
			/// </summary>
            ~Tab()
            {
                Dispose(false);
            }

			/// <summary>
			/// Content
			/// </summary>
            public IDockContent Content
            {
                get { return _content; }
            }

			/// <summary>
			/// Cleanup
			/// </summary>
            public void Dispose()
            {
                Dispose(true);
                GC.SuppressFinalize(this);
            }
			/// <summary>
			/// Cleanup
			/// </summary>
			/// <param name="disposing">bool</param>
            protected virtual void Dispose(bool disposing)
            {
            }
		}
		#endregion

		#region TabCollection
		/// <summary>
		/// TabCollection
		/// </summary>
		[SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible")]
        protected sealed class TabCollection : IEnumerable<Tab>
        {
            #region IEnumerable Members
            IEnumerator<Tab> IEnumerable<Tab>.GetEnumerator()
            {
                for (int i = 0; i < Count; i++)
                    yield return this[i];
            }

            IEnumerator IEnumerable.GetEnumerator()
            {
                for (int i = 0; i < Count; i++)
                    yield return this[i];
            }
            #endregion

            internal TabCollection(DockPane pane)
            {
                _dockPane = pane;
            }

            private DockPane _dockPane = null;

			/// <summary>
			/// DockPane
			/// </summary>
            public DockPane DockPane
            {
                get { return _dockPane; }
            }
			/// <summary>
			/// DockPanel
			/// </summary>
            public DockPanel DockPanel
            {
                get { return DockPane.DockPanel; }
            }
			/// <summary>
			/// Count
			/// </summary>
            public int Count
            {
                get { return DockPane.DisplayingContents.Count; }
            }
			/// <summary>
			/// indexer
			/// </summary>
			/// <param name="index">int</param>
			/// <returns>Tab</returns>
            public Tab this[int index]
            {
                get
                {
                    IDockContent content = DockPane.DisplayingContents[index];
                    if (content == null)
                        throw (new ArgumentOutOfRangeException("index"));
                    if (content.DockHandler.AutoHideTab == null)
                        content.DockHandler.AutoHideTab = (DockPanel.AutoHideStripControl.CreateTab(content));
                    return content.DockHandler.AutoHideTab as Tab;
                }
            }
			/// <summary>
			/// Contains Instance
			/// </summary>
			/// <param name="tab">Instance</param>
			/// <returns>true/false</returns>
            public bool Contains(Tab tab)
            {
                return (IndexOf(tab) != -1);
            }
			/// <summary>
			/// Contains Instance
			/// </summary>
			/// <param name="content">Instance content</param>
			/// <returns>true/false</returns>
            public bool Contains(IDockContent content)
            {
                return (IndexOf(content) != -1);
            }
			/// <summary>
			/// IndexOf
			/// </summary>
			/// <param name="tab">Instance</param>
			/// <returns>index</returns>
            public int IndexOf(Tab tab)
            {
                if (tab == null)
                    return -1;

                return IndexOf(tab.Content);
            }
			/// <summary>
			/// IndexOf
			/// </summary>
			/// <param name="content">Instance content</param>
			/// <returns>index</returns>
            public int IndexOf(IDockContent content)
            {
                return DockPane.DisplayingContents.IndexOf(content);
            }
		}
		#endregion

		#region Pane
		/// <summary>
		/// Pane
		/// </summary>
		[SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible")]
        protected class Pane : IDisposable
        {
            private DockPane _dockPane;

			/// <summary>
			/// ctor
			/// </summary>
			/// <param name="dockPane"></param>
            protected internal Pane(DockPane dockPane)
            {
                _dockPane = dockPane;
            }

			/// <summary>
			/// Finalizer
			/// </summary>
            ~Pane()
            {
                Dispose(false);
            }

			/// <summary>
			/// DockPane
			/// </summary>
            public DockPane DockPane
            {
                get { return _dockPane; }
            }

			/// <summary>
			/// AutoHideTabs
			/// </summary>
            public TabCollection AutoHideTabs
            {
                get
                {
                    if (DockPane.AutoHideTabs == null)
                        DockPane.AutoHideTabs = new TabCollection(DockPane);
                    return DockPane.AutoHideTabs as TabCollection;
                }
            }
			/// <summary>
			/// Cleanup
			/// </summary>
            public void Dispose()
            {
                Dispose(true);
                GC.SuppressFinalize(this);
            }
			/// <summary>
			/// Cleanup
			/// </summary>
			/// <param name="disposing">bool</param>
            protected virtual void Dispose(bool disposing)
            {
            }
		}
		#endregion

		#region PaneCollection
		/// <summary>
		/// PaneCollection
		/// </summary>
		[SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible")]
        protected sealed class PaneCollection : IEnumerable<Pane>
        {
            private class AutoHideState
            {
                public DockState _dockState;
                public bool _selected = false;

                public AutoHideState(DockState dockState)
                {
                    _dockState = dockState;
                }

                public DockState DockState
                {
                    get { return _dockState; }
                }

                public bool Selected
                {
                    get { return _selected; }
                    set { _selected = value; }
                }
            }

            private class AutoHideStateCollection
            {
                private AutoHideState[] _states;

                public AutoHideStateCollection()
                {
                    _states = new AutoHideState[]	{	
												new AutoHideState(DockState.DockTopAutoHide),
												new AutoHideState(DockState.DockBottomAutoHide),
												new AutoHideState(DockState.DockLeftAutoHide),
												new AutoHideState(DockState.DockRightAutoHide)
											};
                }

                public AutoHideState this[DockState dockState]
                {
                    get
                    {
                        for (int i = 0; i < _states.Length; i++)
                        {
                            if (_states[i].DockState == dockState)
                                return _states[i];
                        }
                        throw new ArgumentOutOfRangeException("dockState");
                    }
                }

                public bool ContainsPane(DockPane pane)
                {
                    if (pane.IsHidden)
                        return false;

                    for (int i = 0; i < _states.Length; i++)
                    {
                        if (_states[i].DockState == pane.DockState && _states[i].Selected)
                            return true;
                    }
                    return false;
                }
            }

            internal PaneCollection(DockPanel panel, DockState dockState)
            {
                _dockPanel = panel;
                _states = new AutoHideStateCollection();
                States[DockState.DockTopAutoHide].Selected = (dockState == DockState.DockTopAutoHide);
                States[DockState.DockBottomAutoHide].Selected = (dockState == DockState.DockBottomAutoHide);
                States[DockState.DockLeftAutoHide].Selected = (dockState == DockState.DockLeftAutoHide);
                States[DockState.DockRightAutoHide].Selected = (dockState == DockState.DockRightAutoHide);
            }
			
			/// <summary>
			/// DockPanel
			/// </summary>
            public DockPanel DockPanel
            {
                get { return _dockPanel; }
			} DockPanel _dockPanel;

            private AutoHideStateCollection _states;
            private AutoHideStateCollection States
            {
                get { return _states; }
            }

			/// <summary>
			/// Count
			/// </summary>
            public int Count
            {
                get
                {
                    int count = 0;
                    foreach (DockPane pane in DockPanel.Panes)
                    {
                        if (States.ContainsPane(pane))
                            count++;
                    }

                    return count;
                }
            }
			/// <summary>
			/// Indexer
			/// </summary>
			/// <param name="index">int</param>
			/// <returns>instance</returns>
            public Pane this[int index]
            {
                get
                {
                    int count = 0;
                    foreach (DockPane pane in DockPanel.Panes)
                    {
                        if (!States.ContainsPane(pane))
                            continue;

                        if (count == index)
                        {
                            if (pane.AutoHidePane == null)
                                pane.AutoHidePane = DockPanel.AutoHideStripControl.CreatePane(pane);
                            return pane.AutoHidePane as Pane;
                        }

                        count++;
                    }
                    throw new ArgumentOutOfRangeException("index");
                }
            }
			/// <summary>
			/// Contains
			/// </summary>
			/// <param name="pane">Pane</param>
			/// <returns>true/false</returns>
            public bool Contains(Pane pane)
            {
                return (IndexOf(pane) != -1);
            }
			/// <summary>
			/// Index Of
			/// </summary>
			/// <param name="pane">Pane</param>
			/// <returns>index</returns>
            public int IndexOf(Pane pane)
            {
                if (pane == null)
                    return -1;

                int index = 0;
                foreach (DockPane dockPane in DockPanel.Panes)
                {
                    if (!States.ContainsPane(pane.DockPane))
                        continue;

                    if (pane == dockPane.AutoHidePane)
                        return index;

                    index++;
                }
                return -1;
            }

            #region IEnumerable Members

            IEnumerator<Pane> IEnumerable<Pane>.GetEnumerator()
            {
                for (int i = 0; i < Count; i++)
                    yield return this[i];
            }

            IEnumerator IEnumerable.GetEnumerator()
            {
                for (int i = 0; i < Count; i++)
                    yield return this[i];
            }

            #endregion
		}
		#endregion

		/// <summary>
		/// ctor
		/// </summary>
		/// <param name="panel"></param>
		protected AutoHideStripBase(DockPanel panel)
		{
			_dockPanel = panel;
			_panesTop = new PaneCollection(panel, DockState.DockTopAutoHide);
			_panesBottom = new PaneCollection(panel, DockState.DockBottomAutoHide);
			_panesLeft = new PaneCollection(panel, DockState.DockLeftAutoHide);
			_panesRight = new PaneCollection(panel, DockState.DockRightAutoHide);

			SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
			SetStyle(ControlStyles.Selectable, false);
		}

		/// <summary>
		/// DockPanel
		/// </summary>
		protected DockPanel DockPanel
		{
			get	{	return _dockPanel;	}
		} DockPanel _dockPanel;

		/// <summary>
		/// PanesTop
		/// </summary>
		protected PaneCollection PanesTop
		{
			get	{	return _panesTop;	}
		} PaneCollection _panesTop;

		/// <summary>
		/// PanesBottom
		/// </summary>
		protected PaneCollection PanesBottom
		{
			get	{	return _panesBottom;	}
		} PaneCollection _panesBottom;

		/// <summary>
		/// PanesLeft
		/// </summary>
		protected PaneCollection PanesLeft
		{
			get	{	return _panesLeft;	}
		} PaneCollection _panesLeft;

		/// <summary>
		/// PanesRight
		/// </summary>
		protected PaneCollection PanesRight
		{
			get	{	return _panesRight;	}
		} PaneCollection _panesRight;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="dockState"></param>
		/// <returns></returns>
		protected PaneCollection GetPanes(DockState dockState)
		{
			if (dockState == DockState.DockTopAutoHide)
				return PanesTop;
			else if (dockState == DockState.DockBottomAutoHide)
				return PanesBottom;
			else if (dockState == DockState.DockLeftAutoHide)
				return PanesLeft;
			else if (dockState == DockState.DockRightAutoHide)
				return PanesRight;
			else
				throw new ArgumentOutOfRangeException("dockState");
		}

        internal int GetNumberOfPanes(DockState dockState)
        {
            return GetPanes(dockState).Count;
        }
		/// <summary>
		/// 
		/// </summary>
		protected Rectangle RectangleTopLeft
		{
			get
			{	
				int height = MeasureHeight();
				return PanesTop.Count > 0 && PanesLeft.Count > 0 ? new Rectangle(0, 0, height, height) : Rectangle.Empty;
			}
		}
		/// <summary>
		/// 
		/// </summary>
		protected Rectangle RectangleTopRight
		{
			get
			{
				int height = MeasureHeight();
				return PanesTop.Count > 0 && PanesRight.Count > 0 ? new Rectangle(Width - height, 0, height, height) : Rectangle.Empty;
			}
		}
		/// <summary>
		/// 
		/// </summary>
		protected Rectangle RectangleBottomLeft
		{
			get
			{
				int height = MeasureHeight();
				return PanesBottom.Count > 0 && PanesLeft.Count > 0 ? new Rectangle(0, Height - height, height, height) : Rectangle.Empty;
			}
		}
		/// <summary>
		/// 
		/// </summary>
		protected Rectangle RectangleBottomRight
		{
			get
			{
				int height = MeasureHeight();
				return PanesBottom.Count > 0 && PanesRight.Count > 0 ? new Rectangle(Width - height, Height - height, height, height) : Rectangle.Empty;
			}
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="dockState"></param>
		/// <returns></returns>
		protected internal Rectangle GetTabStripRectangle(DockState dockState)
		{
			int height = MeasureHeight();
			if (dockState == DockState.DockTopAutoHide && PanesTop.Count > 0)
				return new Rectangle(RectangleTopLeft.Width, 0, Width - RectangleTopLeft.Width - RectangleTopRight.Width, height);
			else if (dockState == DockState.DockBottomAutoHide && PanesBottom.Count > 0)
				return new Rectangle(RectangleBottomLeft.Width, Height - height, Width - RectangleBottomLeft.Width - RectangleBottomRight.Width, height);
			else if (dockState == DockState.DockLeftAutoHide && PanesLeft.Count > 0)
				return new Rectangle(0, RectangleTopLeft.Width, height, Height - RectangleTopLeft.Height - RectangleBottomLeft.Height);
			else if (dockState == DockState.DockRightAutoHide && PanesRight.Count > 0)
				return new Rectangle(Width - height, RectangleTopRight.Width, height, Height - RectangleTopRight.Height - RectangleBottomRight.Height);
			else
				return Rectangle.Empty;
		}

		private GraphicsPath _displayingArea = null;
		private GraphicsPath DisplayingArea
		{
			get
			{
				if (_displayingArea == null)
					_displayingArea = new GraphicsPath();

				return _displayingArea;
			}
		}

		private void SetRegion()
		{
			DisplayingArea.Reset();
			DisplayingArea.AddRectangle(RectangleTopLeft);
			DisplayingArea.AddRectangle(RectangleTopRight);
			DisplayingArea.AddRectangle(RectangleBottomLeft);
			DisplayingArea.AddRectangle(RectangleBottomRight);
			DisplayingArea.AddRectangle(GetTabStripRectangle(DockState.DockTopAutoHide));
			DisplayingArea.AddRectangle(GetTabStripRectangle(DockState.DockBottomAutoHide));
			DisplayingArea.AddRectangle(GetTabStripRectangle(DockState.DockLeftAutoHide));
			DisplayingArea.AddRectangle(GetTabStripRectangle(DockState.DockRightAutoHide));
			Region = new Region(DisplayingArea);
		}

		/// <summary>
		/// OnMouseDown
		/// </summary>
		/// <param name="e">MouseEventArgs</param>
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown(e);

			if (e.Button != MouseButtons.Left)
				return;

			IDockContent content = HitTest();
			if (content == null)
				return;

			content.DockHandler.Activate();
		}

		/// <summary>
		/// OnMouseHover
		/// </summary>
		/// <param name="e">EventArgs</param>
		protected override void OnMouseHover(EventArgs e)
		{
			base.OnMouseHover(e);

			IDockContent content = HitTest();
			if (content != null && DockPanel.ActiveAutoHideContent != content)
				DockPanel.ActiveAutoHideContent = content;

			// requires further tracking of mouse hover behavior,
            ResetMouseEventArgs();
		}

		/// <summary>
		/// OnLayout
		/// </summary>
		/// <param name="levent">LayoutEventArgs</param>
		protected override void OnLayout(LayoutEventArgs levent)
		{
			RefreshChanges();
			base.OnLayout (levent);
		}

		internal void RefreshChanges()
		{
            if (IsDisposed)
                return;

			SetRegion();
			OnRefreshChanges();
		}

		/// <summary>
		/// OnRefreshChanges
		/// </summary>
		protected virtual void OnRefreshChanges()
		{
		}

		/// <summary>
		/// MeasureHeight
		/// </summary>
		/// <returns>int</returns>
		protected internal abstract int MeasureHeight();

		private IDockContent HitTest()
		{
			Point ptMouse = PointToClient(Control.MousePosition);
			return HitTest(ptMouse);
		}

		/// <summary>
		/// CreateTab
		/// </summary>
		/// <param name="content">IDockContent</param>
		/// <returns>Tab</returns>
        protected virtual Tab CreateTab(IDockContent content)
        {
            return new Tab(content);
        }

		/// <summary>
		/// CreatePane
		/// </summary>
		/// <param name="dockPane">DockPane</param>
		/// <returns>Pane</returns>
        protected virtual Pane CreatePane(DockPane dockPane)
        {
            return new Pane(dockPane);
        }

		/// <summary>
		/// HitTest
		/// </summary>
		/// <param name="point">Point</param>
		/// <returns>IDockContent</returns>
		protected abstract IDockContent HitTest(Point point);
	}
}