// HSS.Forms.Docking.DockPanel.DragHandler.cs
// ----------------------------------------------------------------------------
// Licensed under the MIT license
// http://www.opensource.org/licenses/mit-license.html
// ----------------------------------------------------------------------------
// HighSpeed-Solutions, LLC
// Copyright (c) 2001-2010
// ----------------------------------------------------------------------------
// File:       DockPanel.DragHandler.cs
// Author:     HSS\gbanta
// Created:    08/12/2010
// Modified:   12/04/2010
// ----------------------------------------------------------------------------
using System.Drawing;
using System.Windows.Forms;

namespace HSS.Forms.Docking
{
	#region DockPanel
	/// <summary>
	/// DockPanel
	/// </summary>
    partial class DockPanel
    {
        /// <summary>
        /// DragHandlerBase is the base class for drag handlers. The derived class should:
        ///   1. Define its public method BeginDrag. From within this public BeginDrag method,
        ///      DragHandlerBase.BeginDrag should be called to initialize the mouse capture
        ///      and message filtering.
        ///   2. Override the OnDragging and OnEndDrag methods.
        /// </summary>
        private abstract class DragHandlerBase : NativeWindow, IMessageFilter
        {
			/// <summary>
			/// DragHandlerBase
			/// </summary>
            protected DragHandlerBase()
            {
            }
			/// <summary>
			/// DragControl
			/// </summary>
            protected abstract Control DragControl
            {
                get;
            }

            
			/// <summary>
			/// StartMousePosition
			/// </summary>
            protected Point StartMousePosition
            {
                get { return _startMousePosition; }
                private set { _startMousePosition = value; }
			} Point _startMousePosition = Point.Empty;

			/// <summary>
			/// BeginDrag
			/// </summary>
			/// <returns></returns>
            protected bool BeginDrag()
            {
                // Avoid re-entrance;
                lock (this)
                {
                    if (DragControl == null)
                        return false;

                    StartMousePosition = Control.MousePosition;

                    if (!NativeMethods.DragDetect(DragControl.Handle, StartMousePosition))
                        return false;

                    DragControl.FindForm().Capture = true;
                    AssignHandle(DragControl.FindForm().Handle);
                    Application.AddMessageFilter(this);
                    return true;
                }
            }
			/// <summary>
			/// OnDragging
			/// </summary>
            protected abstract void OnDragging();
			/// <summary>
			/// OnEndDrag
			/// </summary>
			/// <param name="abort">bool</param>
            protected abstract void OnEndDrag(bool abort);

            private void EndDrag(bool abort)
            {
                ReleaseHandle();
                Application.RemoveMessageFilter(this);
                DragControl.FindForm().Capture = false;

                OnEndDrag(abort);
            }

			/// <summary>
			/// PreFilterMessage
			/// </summary>
			/// <param name="m">Message</param>
			/// <returns>bool</returns>
            bool IMessageFilter.PreFilterMessage(ref Message m)
            {
                if (m.Msg == (int)Msgs.WM_MOUSEMOVE)
                    OnDragging();
                else if (m.Msg == (int)Msgs.WM_LBUTTONUP)
                    EndDrag(false);
                else if (m.Msg == (int)Msgs.WM_CAPTURECHANGED)
                    EndDrag(true);
                else if (m.Msg == (int)Msgs.WM_KEYDOWN && (int)m.WParam == (int)Keys.Escape)
                    EndDrag(true);

                return OnPreFilterMessage(ref m);
            }

			/// <summary>
			/// OnPreFilterMessage
			/// </summary>
			/// <param name="m">Message</param>
			/// <returns>bool</returns>
            protected virtual bool OnPreFilterMessage(ref Message m)
            {
                return false;
            }

			/// <summary>
			/// WndProc
			/// </summary>
			/// <param name="m">Message</param>
            protected sealed override void WndProc(ref Message m)
            {
                if (m.Msg == (int)Msgs.WM_CANCELMODE || m.Msg == (int)Msgs.WM_CAPTURECHANGED)
                    EndDrag(true);

                base.WndProc(ref m);
            }
        }

        private abstract class DragHandler : DragHandlerBase
        {
            private DockPanel _dockPanel;

            protected DragHandler(DockPanel dockPanel)
            {
                _dockPanel = dockPanel;
            }

            public DockPanel DockPanel
            {
                get { return _dockPanel; }
            }

            private IDragSource _dragSource;
            protected IDragSource DragSource
            {
                get { return _dragSource; }
                set { _dragSource = value; }
            }

            protected sealed override Control DragControl
            {
                get { return DragSource == null ? null : DragSource.DragControl; }
            }

            protected sealed override bool OnPreFilterMessage(ref Message m)
            {
                if ((m.Msg == (int)Msgs.WM_KEYDOWN || m.Msg == (int)Msgs.WM_KEYUP) &&
                    ((int)m.WParam == (int)Keys.ControlKey || (int)m.WParam == (int)Keys.ShiftKey))
                    OnDragging();

                return base.OnPreFilterMessage(ref m);
            }
        }
	}
	#endregion
}